CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C                  projet Estime, INRIA-Rocquencourt                  C
C                                                                     C
C  Copyright 2004-2011 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: store_memory_mapping.F,v 1.5 2011-03-10 10:41:56 fclement Exp $

C     
C     Routines to fill the memory mapping.
C     
C     The argument io is either PIO_IN for incoming communications or
C     PIO_OUT for outgoing communications.
C     
      
#include "pio_constants.h"
      
      subroutine store_communication_type(sbr, io, tc)
C     Store tc into the slot SLOT_COMMUNICATION_TYPE (Tc).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, tc
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_COMMUNICATION_TYPE
      
      call rand_store_slot(sbr, io, loc, tc)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', tc
      endif
      
      return
      end
      
      
      subroutine store_communication_values_number(sbr, io, nv)
C     Store nv into the slot SLOT_VALUES_NUMBER (Nv).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, nv
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_VALUES_NUMBER
      
      call rand_store_slot(sbr, io, loc, nv)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', nv
      endif
      
      return
      end
      
      
      subroutine store_communication_name_begin(sbr, io, bn)
C     Set the beginning in memory of the communication name to bn.
C     Store bn into the slot SLOT_NAME_BEGIN (bn).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, bn
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_NAME_BEGIN
      
      call rand_store_slot(sbr, io, loc, bn)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', bn
      endif
      
      return
      end
      
      
      subroutine store_communication_name_end(sbr, io, en)
C     Set the end in memory of the communication name to en.
C     Store en into the slot SLOT_NAME_END (en).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, en
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_NAME_END
      
      call rand_store_slot(sbr, io, loc, en)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', en
      endif
      
      return
      end
      
      
      subroutine store_communication_begin(sbr, io, bc)
C     Set the beginning in memory of the communication values to bc.
C     Store bc into the slot SLOT_COMMUNICATION_BEGIN (bc).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, bc
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_COMMUNICATION_BEGIN
      
      call rand_store_slot(sbr, io, loc, bc)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', bc
      endif
      
      return
      end
      
      
      subroutine store_communication_end(sbr, io, ec)
C     Set the end in memory of the communication values to ec.
C     Store ec into the slot SLOT_COMMUNICATION_END (ec).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, ec
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_COMMUNICATION_END
      
      call rand_store_slot(sbr, io, loc, ec)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', ec
      endif
      
      return
      end
      
      
      subroutine store_communication_value_descriptor_address
     +     (sbr, io, i, add)
C     Set the value descriptor address of the current value (ith) to
C     add.
C     Set item i of slot vda to add.
C     Store add into slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, i, add
      
C     Local variable.
      integer loc
      
C     External function.
      logical pio_get_debug_mode
      
      loc = SLOT_VALUE_DESCRIPTOR_ADDRESS(i)
      
      call rand_store_slot(sbr, io, loc, add)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) 'In ', sbr, ', ',
     +        'setting memory mapping slot ', loc, ' to ', add
      endif
      
      return
      end
      
      
      subroutine store_value_type(sbr, io, tv)
C     Set the type of the current value (ith) to tv.
C     Store tv into slot SLOT_VALUE_TYPE of item i of slot vm.
C     Store tv into slot (vda(i) + SLOT_VALUE_TYPE), where vda(i) is
C     the value in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, tv
      
      call value_descriptor_relative_store_slot
     +     (sbr, io, SLOT_VALUE_TYPE, tv)
      
      return
      end
      
      
      subroutine store_value_dimension(sbr, io, pv)
C     Set the dimension of the current value (ith) to pv.
C     Store pv into slot SLOT_VALUE_DIMENSION of item i of slot vm.
C     Store pv into slot (vda(i) + SLOT_VALUE_DIMENSION), where vda(i)
C     is the value in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, pv
      
      call value_descriptor_relative_store_slot
     +     (sbr, io, SLOT_VALUE_DIMENSION, pv)
      
      return
      end
      
      
      subroutine store_value_begin(sbr, io, bv)
C     Set the beginning in memory of the current value (ith) to bv.
C     Store bv into slot SLOT_VALUE_BEGIN of item i of slot vm.
C     Store bv into slot (vda(i) + SLOT_VALUE_BEGIN), where vda(i) is
C     the value in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, bv
      
      call value_descriptor_relative_store_slot
     +     (sbr, io, SLOT_VALUE_BEGIN, bv)
      
      return
      end
      
      
      subroutine store_value_end(sbr, io, ev)
C     Set the end in memory of the current value (ith) to ev.
C     Store ev into slot SLOT_VALUE_END of item i of slot vm.
C     Store ev into slot (vda(i) + SLOT_VALUE_END), where vda(i) is
C     the value in slot (SLOT_VALUE_DESCRIPTOR_ADDRESSES + i - 1).
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, ev
      
      call value_descriptor_relative_store_slot
     +     (sbr, io, SLOT_VALUE_END, ev)
      
      return
      end
      
      
      subroutine store_value_size(sbr, io, p, sizev)
C     Set the size of the current value (ith) to sizev.
C     Store sizev into the p current slots of item i of slot vm.
C     Store sizev into slots
C     store_value_descriptor_ptr..(store_value_descriptor_ptr+p-1)
C     and increment store_value_descriptor_ptr of p.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, p, sizev(*)
      
C     Local variable.
      integer i
      
      do i = 1, p
         call seq_store_slot(sbr, io, sizev(i))
      enddo
      
      return
      end
      
      
      subroutine store_value_layout_kind(sbr, io, lkv)
C     Set the layout kind of the current value (ith) to lkv.
C     Store lkv into the current slot of item i of slot vm.
C     Store lkv into slot store_value_descriptor_ptr and increment
C     store_value_descriptor_ptr.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, lkv
      
      call seq_store_slot(sbr, io, lkv)
      
      return
      end
      
      
      subroutine store_value_layout(sbr, io, p, layoutv)
C     Set the layout of the current value (ith) to layoutv.
C     Store layoutv into the p current slots of item i of slot vm.
C     Store layoutv into slots
C     store_value_descriptor_ptr..(store_value_descriptor_ptr+p-1)
C     and increment store_value_descriptor_ptr of p.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, p, layoutv(MAX_ARRAY_DIMENSION)
      
C     Local variable.
      integer i
      
      do i = 1, p
         call seq_store_slot(sbr, io, layoutv(i))
      enddo
      
      return
      end
      
      
      subroutine store_value_len(sbr, io, lenv)
C     Set the length of the current item of the current value (ith) to
C     lenv.
C     Store lenv into the current slot of item i of slot vm.
C     Store lenv into slot store_value_descriptor_ptr and increment
C     store_value_descriptor_ptr.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, lenv
      
      call seq_store_slot(sbr, io, lenv)
      
      return
      end
